### SHELF 3.0 - Dirichlet distribution 
### Last update: Tue Mar 12 08:14:52 BRT 2013
### Leo Bastos - lsbastos @ fiocruz . br


###  Auxiliary functions for eliciting a Dirichlet distrbution
### It needs some output from SHELF 2

####  After eliciting the marginal Beta distribution for all parameters 
#### it is necessary a data.frame with all hyper parameters.

# D.hyper = data.frame(d = c(d1, d2,...,dk), e = c(e1, e2,...,ek) )

constraint.marginals = function( d, e ){
  out = sum(mean.beta(d,e))
  if(out < 0.9 | out > 1.1) warning("Dirichlet distribution may not represent the uncertainty using these values for d and e.")
  return(out)
}


# mean vector
mean.beta = function(d, e){
  d / (d + e)
}

var.beta = function(d, e){
  Var = d * e / ( (d + e)^2 * (d + e + 1) )
  Var
}

quartiles.beta = function(d, e){
  quart = matrix(0, nc=length(d), nr=3)
  rownames(quart) = c("Q1","Q2", "Q3")
  
  k = 1
  for( q in c(.25, 0.50, .75)){
    quart[k,] = qbeta(q, d, e)
    k = k + 1
  }
  
  quart
}

# Marginal SD
sd.beta = function(d, e){
  sqrt(var.beta(d,e))
}

# Adjusted SD
adj.SD = function(n, mean.mar){
  d.n = n * mean.mar
  Var = d.n * (n - d.n) / (n^2 * (n+1))
  sqrt(Var)
}


# Objetive function used to find the optimal central value
Delta.dirichlet = function(n, d, e){
  origSD = sd.beta(d,e)
  mean.beta = mean.beta(d, e)
  adjSD = adj.SD(n=n, mean.mar=mean.beta)
  
  drop(sqrt(crossprod(origSD-adjSD)))
}

optimal.n = function(d, e){
  # Analytical solution of the optimal n
  # n.opt = arg min ( sum_k ( sd_k(d,e) - sd^{ADJ}_{k}(n,d,e) )^2 )
  vk = var.beta(d,e)
  n.opt = sum( vk * (d + e + 1) ) / sum( vk * sqrt(d + e + 1) )
  n.opt = n.opt^2 - 1
  return(n.opt)
}

